import argparse

# Read input strings
argParser = argparse.ArgumentParser()
argParser.add_argument("-in", "--infile", help="name of the input file")
argParser.add_argument("-out", "--outfile", help="name of the output file")
args = argParser.parse_args()


# Input: Two strings
# Output: Hamming distance between those two stings
def hamming_distance_2(string1, string2):
    if not len(string1) == len(string2):
        raise TypeError(f'Strings "{string1}" and "{string2}" do not have the same length')
    length = len(string1)
    dist = 0
    for i in range(length):
        if not string1[i] == string2[i]:
            dist += 1
    return dist


# Input: Array of strings and string s
# Output: Returns the maximum hamming distance between s and the strings in the array
def hamming_distance(strings, s):
    num_strings = len(strings)
    max_dist = 0
    for i in range(num_strings):
        try:
            curr_dist = hamming_distance_2(strings[i], s)
        except ValueError:
            raise
        max_dist = max(max_dist, curr_dist)
    return max_dist


if __name__ == '__main__':
    # INFO: This heuristic might not be the best, since it starts with a random string an then just
    # changes order of chars. Number of each char stays the same.

    # Read input strings
    if args.infile is None:
        raise TypeError(f'You need to give the name of a file with input strings')

    strings = []

    f = open(args.infile, "r")
    for line in f:
        strings.append(line.strip())
    f.close()

    N = len(strings)
    L = len(strings[0])

    cs = ""
    for n in range(L):
        listOfChars = []
        for string in strings:
            listOfChars.append(string[n])
        # Note: When there is a tie, then the "A" is more likely to be choosen; Change that
        cs += max(sorted(set(listOfChars)), key=listOfChars.count)

    # Write output
    if args.outfile is None:
        outfile = "greedy_output.txt"
    else:
        outfile = args.outfile

    with open(outfile, 'w') as f:
        f.write(cs)
        f.write("\n\n")
        f.write("hamming distance: ")
        f.write(str(hamming_distance(strings, cs)))

    print("The current closest string with hamming distance of", str(hamming_distance(strings, cs)), "was written into the file", outfile)